#ifndef CMATHS_H
#define CMATHS_H

#include <stdint.h>
#include <math.h>

#define MAX(_a, _b) ((_a) > (_b) ? (_a) : (_b))
#define MIN(_a, _b) ((_a) < (_b) ? (_a) : (_b))
#define ABS(_a) ((_a) >= 0 ? (_a) : -(_a))
#define SGN(_a) ( (_a) < 0 ? -1 : \
                 ((_a) > 0 ? 1 : 0))

//---------------------------------------------------------------------------------------------
class CVec2
{
public:
    CVec2(int32_t px = 0, int32_t py = 0);

    CVec2 operator+(const CVec2& rhs) const;
    CVec2 operator-(const CVec2& rhs) const;
    CVec2 operator/(int32_t rhs) const;
    CVec2 operator/(float rhs) const;
    CVec2 operator*(int32_t rhs) const;
    CVec2 operator*(float rhs) const;
    CVec2 operator%(int32_t rhs) const;
    void operator+=(const CVec2& rhs);
    void operator-=(const CVec2& rhs);
    void operator/=(int32_t rhs);
    void operator*=(int32_t rhs);
    void operator*=(float rhs);
    bool operator==(const CVec2& rhs);
    bool operator!=(const CVec2& rhs);
    uint32_t squaredLength();
    float    length();
    static CVec2 abs(const CVec2& vec);
    static CVec2 sgn(const CVec2& vec);
    uint32_t manhattanDistance(const CVec2& dest);

    int32_t x;
    int32_t y;
};

//---------------------------------------------------------------------------------------------
class CRect
{
public:
    CRect();
    CRect(const CVec2& pTL, const CVec2& pBR);

    CVec2 tl;   // top/left
    CVec2 br;   // bottom/right
};

#endif // CMATHS_H
