#ifndef TOOLS_H
#define TOOLS_H

#include <QUndoCommand>
#include "../common/sources/system/cmaths.h"
#include "../common/sources/Map.h"
#include "editor.h"
#include "clipboard.h"

//-------------------------------------------------------------------
class drawTileTool : public QUndoCommand
{
public:
    explicit drawTileTool(QUndoCommand *parent = 0);

    bool init(CVec2 pos, int type);
    void undo() override;
    void redo() override;

private:
    CVec2   mPos;
    int     mType;
    CTile   mOldTile;
};

//-------------------------------------------------------------------
class drawWallTool : public QUndoCommand
{
public:
    explicit drawWallTool(QUndoCommand *parent = 0);

    bool init(CVec2 pos, int type);
    void undo() override;
    void redo() override;

private:
    CVec2   mPos;
    int     mType;
    CWall   mOldWall;
    EWallSide mSide;
};

//-------------------------------------------------------------------
class addObjectTool : public QUndoCommand
{
public:
    explicit addObjectTool(QUndoCommand *parent = 0);

    bool init(CVec2 pos, int type);
    void undo() override;
    void redo() override;

private:
    CVec2   mPos;
    int     mType;
};

//-------------------------------------------------------------------
class drawMonsterTool : public QUndoCommand
{
public:
    explicit drawMonsterTool(QUndoCommand *parent = 0);

    bool init(CVec2 pos, int type);
    void undo() override;
    void redo() override;

private:
    CVec2       mPos;
    int         mType;
    CMonsterGroup   mOldMonster;
};

//-------------------------------------------------------------------
class cutTool : public QUndoCommand
{
public:
    explicit cutTool(QUndoCommand *parent = 0);

    bool init(CVec2 mousePos, ETabIndex tabIndex);
    void undo() override;
    void redo() override;

private:
    CVec2       mPos;
    CClipboard  mData;
    ETabIndex   mTabIndex;
};

//-------------------------------------------------------------------
class pasteTool : public QUndoCommand
{
public:
    explicit pasteTool(QUndoCommand *parent = 0);

    bool init(CVec2 mousePos, ETabIndex tabIndex);
    void undo() override;
    void redo() override;

private:
    CVec2       mPos;
    CClipboard  mData;
    CClipboard  mClipboard;
    ETabIndex   mTabIndex;
};

#endif // TOOLS_H
