#include "bridge.h"
#include "game.h"
#include <time.h>
#include <QDebug>

CBridge     bridge;

CBridge::CBridge(QObject *parent) : QObject(parent)
{
    mMousePos = CVec2();
    mMouseButton = false;
}

void    CBridge::setMouse(qint32 x, qint32 y, qint32 windowWidth, qint32 windowHeight)
{
    CVec2   imagePos;
    CVec2   imageSize;
    float widthScale = (float)windowWidth / (float)SCREEN_WIDTH;
    float heightScale = (float)windowHeight / (float)SCREEN_HEIGHT;

    if (widthScale <= heightScale)
    {
        imageSize.x = (int32_t)windowWidth;
        imageSize.y = (int32_t)(widthScale * (float)SCREEN_HEIGHT);
        imagePos.x = 0;
        imagePos.y = (windowHeight - imageSize.y) / 2;
    }
    else
    {
        imageSize.x = (int32_t)(heightScale * (float)SCREEN_WIDTH);
        imageSize.y = (int32_t)windowHeight;
        imagePos.x = (windowWidth - imageSize.x) / 2;
        imagePos.y = 0;
    }

    mMousePos = CVec2(x, y) - imagePos;
    mMousePos.x = (mMousePos.x * SCREEN_WIDTH) / imageSize.x;
    mMousePos.y = (mMousePos.y * SCREEN_HEIGHT) / imageSize.y;
}

void    CBridge::setMousePress(qint32 button)
{
    if (button == Qt::LeftButton)
        mMouseButton = true;
}

void    CBridge::setMouseRelease(qint32 button)
{
    if (button == Qt::LeftButton)
        mMouseButton = false;
}

//---------------------------------------------------------------------------------------------
MyImageProvider::MyImageProvider() : QQuickImageProvider(QQuickImageProvider::Image)
{
}

QImage MyImageProvider::requestImage(const QString &id, QSize *size, const QSize &requestedSize)
{
/*    static clock_t  lastClock;
    clock_t  curClock = clock();
    qDebug() << (((float)(curClock - lastClock)) * 1000.0f /CLOCKS_PER_SEC);
    lastClock = curClock;
*/

    QImage& image = game.mainLoop();

    if (size != NULL)
        *size = QSize(image.width(), image.height());

    return image;
}
