#ifndef MONSTERS_H
#define MONSTERS_H

#include <QString>
#include <QImage>
#include <vector>
#include "../common/sources/Map.h"
#include "timer.h"

//---------------------------------------------------------------------------------------------
enum EMonsterPos
{
    eMonsterPosNone,    // empty monster
    eMonsterPosNorthWest,
    eMonsterPosNorthEast,
    eMonsterPosSouthWest,
    eMonsterPosSouthEast,
    eMonsterPosCenter
};

enum EMonsterDir
{
    eMonsterDirUp,
    eMonsterDirDown,
    eMonsterDirLeft,
    eMonsterDirRight
};

enum EMonsterState
{
    eMonsterState_Wandering,
    eMonsterState_Fight,
    eMonsterState_Seek
};

#define MONSTER_TYPE_SCREAMER   7

class CMonsterGroup2;

//---------------------------------------------------------------------------------------------
class CMonster
{
public:
    void    update(CMonsterGroup2& group, CVec2 mapPos, uint8_t type, bool isInFront);
    void    wander();
    void    fight(CVec2 mapPos, uint8_t type, bool isInFront);
    void    seek();
    void    initNextAttack(uint8_t type);
    uint8_t chooseTarget(CVec2 mapPos);
    void    load(FILE* handle);
    void    save(FILE* handle);

    EMonsterPos pos;
    EMonsterDir dir;
    int life;
    CTimer  displayAttackTimer;
    CTimer  nextAttackTimer;
};

//---------------------------------------------------------------------------------------------
class CMonsterGroup2
{
public:
    CMonsterGroup2();
    void    initTimers(uint8_t type);
    void    update(CVec2 mapPos, uint8_t type);
    void    wander(CVec2 mapPos, uint8_t type);
    void    fight(CVec2 mapPos, uint8_t type);
    void    seek(CVec2 mapPos, uint8_t type);
    void    hitMonster(CVec2 groupPos, int monsterNum, int damages);
    void    load(FILE* handle, uint8_t type);
    void    save(FILE* handle);
    void    enterWanderState();
    void    enterFightState(CVec2 mapPos);
    void    enterSeekState();

    uint8_t getMonsterRow(CVec2 mapPos, EMonsterPos monsterPos);
    bool    isMonsterInFront(CVec2 mapPos, EMonsterPos monsterPos);

    CMonster        mMonsters[4];
    EMonsterDir     dir;
    EMonsterState   state;
    CVec2           startPos;
    CTimer          moveTimer;

private:
    CVec2           lastPlayerPos;

    CVec2   getNewPos(CVec2 pos, EWallSide side);
    bool    canMove(CVec2 pos, EWallSide side);
    void    move(CVec2 mapPos, uint8_t type);
    void    initMoveTimer(int type);
    int     getVisibleDistance(CVec2 pos, CVec2 target, uint8_t type);
    bool    isTargetInConeOfVision(CVec2 pos, CVec2 target, EMonsterDir dir);
    int     distanceWithWalls(CVec2 source, CVec2 dest);
    bool    isWallBetweenPos(CVec2 source, CVec2 dest);
    EMonsterDir nextDir(EMonsterDir dir);
    EMonsterDir prevDir(EMonsterDir dir);
    EMonsterDir oppositeDir(EMonsterDir dir);
    EMonsterDir getDirectionToTarget(CVec2 pos, CVec2 target, EMonsterDir& secondaryDir);
};

//---------------------------------------------------------------------------------------------
class CMonsters
{
public:
    struct SMonsterData
    {
        QString frontImage;
        QString sideImage;
        QString backImage;
        QString attackImage;
        QString attackSound;
        QString moveSound;
        int     attackDisplayTime;
        int     timeBetweenAttack;
        int     movementTime;
        int     defense;
        int     baseHealth;
        int     attack;
        int     dexterity;
        bool    sideAttack;
        int     sightRange;
    };

    CMonsters();
    void    readMonstersDB();
    void    init();
    void    draw(QImage* image, CVec2 mapPos, CVec2 tablePos, int row);
    void    update();
    CVec2   getMonsterPos2(CVec2 tablePos, EMonsterPos monsterPos);
    static CVec2   getMonsterPos(float x, float y);

    std::vector<SMonsterData>   monstersDatas;
    std::vector<CMonsterGroup2> monsterGroups;

private:
    int     getStartingLife(int type);
};

extern CMonsters    monsters;
extern const float monsterPosDir[4][5][2];

#endif // MONSTERS_H
